(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('tslib'), require('rxjs/internal/operators')) :
    typeof define === 'function' && define.amd ? define('@webdev-tools/ng-nested-reactive-forms', ['exports', '@angular/core', '@angular/forms', 'tslib', 'rxjs/internal/operators'], factory) :
    (factory((global['webdev-tools'] = global['webdev-tools'] || {}, global['webdev-tools']['ng-nested-reactive-forms'] = {}),global.ng.core,global.ng.forms,global.tslib,null));
}(this, (function (exports,core,forms,tslib,operators) { 'use strict';

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    /**
     * A component to abstract the form implementation
     *
     * It holds the validation and the submit logic
     *
     * \@example
     * <form nrfForm (nrfSubmit)="aCallback($event)">
     *  // Inputs
     * </form>
     */
    var NrfFormDirective = (function () {
        /**
         * @ignore
         */
        function NrfFormDirective() {
            /**
             * A function that will be called when the form is valid and a submit event is triggered
             * by a button click or programmatically.
             */
            this.nrfSubmit = new core.EventEmitter();
            this.formGroup = new forms.FormGroup({});
        }
        /**
         * Init an empty nrfEntity, in case none where provided.
         * @return {?}
         */
        NrfFormDirective.prototype.ngOnInit = function () {
            if (!this.nrfEntity) {
                this.nrfEntity = {};
            }
            this.formData = this.cloneDeep(this.nrfEntity);
        };
        /**
         * @return {?}
         */
        NrfFormDirective.prototype.ngOnDestroy = function () {
            this.nrfSubmit.complete();
        };
        /**
         * # Should not be called directly!
         * This method wraps the form validation and call [onSubmit]{\@link NrfFormDirective#nrfSubmit}
         * @param {?} $event
         * @return {?}
         */
        NrfFormDirective.prototype.formSubmitWrapper = function ($event) {
            $event.preventDefault();
            if (!this.formGroup.valid) {
                return;
            }
            this.nrfSubmit.emit({
                nrfForm: this,
                formData: this.formData,
                entity: this.nrfEntity,
                formGroup: this.formGroup,
                event: $event,
            });
        };
        /**
         * @param {?} target
         * @return {?}
         */
        NrfFormDirective.prototype.cloneDeep = function (target) {
            var _this = this;
            if (!target || typeof target !== 'object') {
                return target;
            }
            if (Array.isArray(target)) {
                return target.map(this.cloneDeep);
            }
            return Object.keys(target).reduce(function (props, key) {
                props[key] = _this.cloneDeep(target[key]);
                return props;
            }, {});
        };
        return NrfFormDirective;
    }());
    NrfFormDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'form[nrfForm]',
                    exportAs: 'nrfForm',
                },] },
    ];
    /** @nocollapse */
    NrfFormDirective.ctorParameters = function () { return []; };
    NrfFormDirective.propDecorators = {
        "nrfEntity": [{ type: core.Input },],
        "nrfSubmit": [{ type: core.Output },],
        "formSubmitWrapper": [{ type: core.HostListener, args: ['submit', ['$event'],] },],
    };
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    /**
     * This module only handle form Abstraction and submit handler
     * No inputs are provided by this module.
     */
    var NrfFormModule = (function () {
        function NrfFormModule() {
        }
        return NrfFormModule;
    }());
    NrfFormModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        forms.FormsModule,
                        forms.ReactiveFormsModule,
                    ],
                    exports: [
                        forms.FormsModule,
                        forms.ReactiveFormsModule,
                        NrfFormDirective,
                    ],
                    declarations: [
                        NrfFormDirective,
                    ],
                },] },
    ];
    /**
     * Class to abstract get and set values from an object or an array
     * using dot-notation string as the path to the property
     *
     */
    var NrfModelSetterService = (function () {
        function NrfModelSetterService() {
            var _this = this;
            /**
             * Get the final property of the dot notation path.
             * This will be the property that will receive the new value
             */
            this.getTargetPropToSet = function (obj, key, i, pathPieces) {
                var /** @type {?} */ nextKey = pathPieces[i + 1];
                var /** @type {?} */ isLast = nextKey == null;
                nextKey = nextKey || pathPieces['finalKey'];
                var /** @type {?} */ isArrayKey = _this.isArrayKey(nextKey);
                var /** @type {?} */ prop = obj[key];
                if (!prop) {
                    prop = isArrayKey ? [] : {};
                    obj[key] = prop;
                }
                else if (isLast) {
                    prop = isArrayKey ? tslib.__spread(prop) : Object.assign({}, prop);
                    obj[key] = prop;
                }
                return prop;
            };
        }
        /**
         * Get a value using a dot-notation string to find the nested property
         *
         * @param {?} path Dot notation of the value to be get
         * @param {?} model An object or an Array
         * @param {?=} separator An optional param to specify a different separator, default '.'
         *
         * ```typescript
         * const nrfEntity = {
         *    user: {
         *        name: 'John Doe'
         *    }
         * };
         * nestedProps.getValue('user.name', nrfEntity);
         * // => 'John Doe'
         *
         * nestedProps.getValue('user.email', nrfEntity);
         * // => null
         * ```
         * @return {?}
         */
        NrfModelSetterService.prototype.getValue = function (path, model, separator) {
            if (separator === void 0) {
                separator = '.';
            }
            if (path == null || !model) {
                return null;
            }
            try {
                var /** @type {?} */ pathPieces = this.generatePathPieces(path, separator);
                return pathPieces.reduce(this.piecesReducer, model) || null;
            }
            catch (err) {
                return null;
            }
        };
        /**
         * @ignore
         *
         * Return the value associated to the given key.
         *
         * If the key is empty, the last element of an array is returned
         * @param {?} obj
         * @param {?} key
         * @return {?}
         */
        NrfModelSetterService.prototype.piecesReducer = function (obj, key) {
            return key === '' ? obj.slice(-1)[0] : obj[key];
        };
        /**
         * Set a value using a dot-notation string.
         *
         * If the property does not exists, the entire path will be created.
         *
         * @param {?} path Dot notation of the value to be get
         * @param {?} value The value to be set
         * @param {?} model An object or an Array
         * @param {?=} separator An optional param to specify a different separator, default '.'
         *
         * ```typescript
         * nestedProps.setValue('user.age', 35, { user: { name: 'John Doe' } });
         * ```
         * @return {?}
         */
        NrfModelSetterService.prototype.setValue = function (path, value, model, separator) {
            if (separator === void 0) {
                separator = '.';
            }
            if (!(path && model)) {
                return null;
            }
            try {
                var /** @type {?} */ pathPieces = this.generatePathPieces(path, separator);
                var /** @type {?} */ finalKey = pathPieces.pop();
                ((pathPieces))['finalKey'] = finalKey;
                var /** @type {?} */ targetProp = pathPieces.reduce(this.getTargetPropToSet, model) || null;
                if (finalKey === '') {
                    targetProp.push(value);
                }
                else {
                    targetProp[finalKey] = value;
                }
            }
            catch (err) {
                console.error('NestedProps setValue error: ', err);
            }
        };
        /**
         * @ignore
         * Internal function to define if a given key is for an array or an object
         * @param {?} key
         * @return {?}
         */
        NrfModelSetterService.prototype.isArrayKey = function (key) {
            return key === '' || !isNaN(parseInt(key, 10));
        };
        /**
         * Generate the dot-notation path as an Array to iterate over.
         * @param {?} path
         * @param {?} separator
         * @return {?}
         */
        NrfModelSetterService.prototype.generatePathPieces = function (path, separator) {
            return path
                .replace('[', separator)
                .replace(']', '')
                .split(separator);
        };
        return NrfModelSetterService;
    }());
    NrfModelSetterService.decorators = [
        { type: core.Injectable },
    ];
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    var NrfNestedControlContext = (function () {
        /**
         * @param {?} formControl
         * @param {?} formGroup
         * @param {?} nrfNestedControl
         */
        function NrfNestedControlContext(formControl, formGroup, nrfNestedControl) {
            this.formControl = formControl;
            this.formGroup = formGroup;
            this.nrfNestedControl = nrfNestedControl;
            this.$implicit = formControl;
        }
        return NrfNestedControlContext;
    }());
    /**
     * This directive control nested inputs and sets values on the Original Model set at {\@link NrfFormDirective#nrfEntity}
     *
     * #### Given an nrfEntity on the controller:
     * ```typescript
     * this.userModel = {
     *    firstName: 'John',
     *    address: {
     *        street: 'Carnaby Street'
     *    }
     * };
     * ```
     *
     * #### Use it on the form
     * ```html
     * <form [nrfEntity]="userModel">
     *   <div class="form-group">
     *      <label for="name">Name:</label>
     *
     *      <input
     *        id="name"
     *        class="form-control"
     *        nrfModel="userModel.name" // TODO fix this example
     *      />
     *   </div>
     * </form>
     * ```
     */
    var NrfNestedControlDirective = (function () {
        /**
         * @param {?} modelSetter
         * @param {?} nrfForm
         * @param {?} templateRef
         * @param {?} viewContainerRef
         * @param {?} __4
         */
        function NrfNestedControlDirective(modelSetter, nrfForm, templateRef, viewContainerRef, _a) {
            var nativeElement = _a.nativeElement;
            this.modelSetter = modelSetter;
            this.nrfForm = nrfForm;
            this.templateRef = templateRef;
            this.viewContainerRef = viewContainerRef;
            this.isRegisteredToFormControl = false;
            this.isDestroyed = false;
            this.inputEl = nativeElement;
        }
        /**
         * Register this input component to its parent form [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
         *
         * And starts to emit the input's value when it changes.
         * @return {?}
         */
        NrfNestedControlDirective.prototype.ngOnInit = function () {
            this.modelPath = this.getModelPathWithoutFirstPart();
            this.registerToFormGroup();
            this.subscribeToValueChanges();
            this.showViewContent();
        };
        /**
         * @return {?}
         */
        NrfNestedControlDirective.prototype.ngOnDestroy = function () {
            this.isDestroyed = true;
            if (this.parentFormGroup) {
                this.parentFormGroup.removeControl(this.nrfModelName);
            }
        };
        /**
         * @return {?}
         */
        NrfNestedControlDirective.prototype.showViewContent = function () {
            var /** @type {?} */ context = new NrfNestedControlContext(this.formControl, this.parentFormGroup, this);
            this.viewContainerRef.createEmbeddedView(this.templateRef, context);
        };
        /**
         * Register this input to its parent [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
         * to enable validations and data manipulation
         * @return {?}
         */
        NrfNestedControlDirective.prototype.registerToFormGroup = function () {
            if (this.isRegisteredToFormControl) {
                return;
            }
            var /** @type {?} */ formGroup = this.getFormGroup();
            var /** @type {?} */ formControl = this.getFormControl(formGroup);
            this.formControl = formControl;
            this.inputEl.value = formControl.value || '';
            if (formGroup) {
                this.isRegisteredToFormControl = true;
                this.parentFormGroup = formGroup;
                formGroup.addControl(this.nrfModelName, formControl);
            }
        };
        /**
         * Verify if this input is inside a [NrfFormDirective]{\@link NrfFormDirective}
         * and return its [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
         *
         * Otherwise a new empty [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
         * @return {?}
         */
        NrfNestedControlDirective.prototype.getFormGroup = function () {
            if (this.nrfForm) {
                return this.nrfForm.formGroup;
            }
            return new forms.FormGroup({});
        };
        /**
         * Verify if the [FormGroup]{\@link https://angular.io/api/forms/FormGroup} has an control with the current name and return it.
         * Otherwise return a new [FormControl]{\@link https://angular.io/api/forms/FormControl}
         * @param {?} formGroup
         * @return {?}
         */
        NrfNestedControlDirective.prototype.getFormControl = function (formGroup) {
            var /** @type {?} */ formControl = formGroup && /** @type {?} */ (formGroup.get(this.nrfModelName));
            if (!formControl) {
                formControl = this.getNewFormControl();
            }
            return formControl;
        };
        /**
         * Instantiate a new [FormControl]{\@link https://angular.io/api/forms/FormControl} and return it
         * @return {?}
         */
        NrfNestedControlDirective.prototype.getNewFormControl = function () {
            var /** @type {?} */ initialValue = this.getInitialValue();
            return new forms.FormControl(initialValue || null);
        };
        /**
         * Subscribe to [valueChanges]{\@link https://angular.io/api/forms/AbstractControl#valueChanges} and update the Entity value
         * @return {?}
         */
        NrfNestedControlDirective.prototype.subscribeToValueChanges = function () {
            var _this = this;
            this.formControl.valueChanges
                .pipe(operators.takeWhile(function () { return !_this.isDestroyed; }))
                .subscribe(function (newValue) { return _this.setModelValue(newValue); });
        };
        /**
         * Return the dot notation path of the nrfModelData, without the first part,
         * because it is the rfModelData itself.
         * @return {?}
         */
        NrfNestedControlDirective.prototype.getModelPathWithoutFirstPart = function () {
            return this.nrfModelName.substr(this.nrfModelName.indexOf('.') + 1);
        };
        /**
         * Get the value from the [nrfEntity]{\@link NrfFormDirective#nrfEntity}
         * @return {?}
         */
        NrfNestedControlDirective.prototype.getInitialValue = function () {
            return this.modelSetter.getValue(this.modelPath, this.nrfForm.nrfEntity);
        };
        /**
         * Set the value to the [formData]{\@link NrfFormDirective#formData}
         * @param {?} newValue
         * @return {?}
         */
        NrfNestedControlDirective.prototype.setModelValue = function (newValue) {
            return this.modelSetter.setValue(this.modelPath, newValue || '', this.nrfForm.formData);
        };
        /**
         * @param {?} input
         * @return {?}
         */
        NrfNestedControlDirective.prototype.onModelChange = function (input) {
            this.formControl.setValue(input.value, { emitModelToViewChange: false });
        };
        return NrfNestedControlDirective;
    }());
    NrfNestedControlDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[nrfNestedControl]',
                    exportAs: 'nrfNestedControl',
                },] },
    ];
    /** @nocollapse */
    NrfNestedControlDirective.ctorParameters = function () {
        return [
            { type: NrfModelSetterService, },
            { type: NrfFormDirective, decorators: [{ type: core.Optional },] },
            { type: core.TemplateRef, },
            { type: core.ViewContainerRef, },
            { type: core.ElementRef, },
        ];
    };
    NrfNestedControlDirective.propDecorators = {
        "nrfModelName": [{ type: core.Input, args: ['nrfNestedControl',] },],
        "onModelChange": [{ type: core.HostListener, args: ['input', ['$event.target'],] },],
    };
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    /**
     * Holds the nrfModel directive implementation
     */
    var NrfModelModule = (function () {
        function NrfModelModule() {
        }
        return NrfModelModule;
    }());
    NrfModelModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [],
                    providers: [
                        NrfModelSetterService,
                    ],
                    declarations: [
                        NrfNestedControlDirective,
                    ],
                    exports: [
                        NrfNestedControlDirective,
                    ],
                },] },
    ];
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    var NrFormsModule = (function () {
        function NrFormsModule() {
        }
        return NrFormsModule;
    }());
    NrFormsModule.decorators = [
        { type: core.NgModule, args: [{
                    exports: [
                        NrfFormModule,
                        NrfModelModule,
                    ]
                },] },
    ];

    exports.NrFormsModule = NrFormsModule;
    exports.NrfFormDirective = NrfFormDirective;
    exports.NrfNestedControlContext = NrfNestedControlContext;
    exports.NrfNestedControlDirective = NrfNestedControlDirective;
    exports.NrfModelSetterService = NrfModelSetterService;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=webdev-tools-ng-nested-reactive-forms.umd.js.map
