/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
import { Directive, ElementRef, HostListener, Input, Optional, TemplateRef, ViewContainerRef } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { NrfModelSetterService } from './model-setter.service';
import { NrfFormDirective } from '../form/form.directive';
import { takeWhile } from 'rxjs/internal/operators';
var NrfNestedControlContext = /** @class */ (function () {
    /**
     * @param {?} formControl
     * @param {?} formGroup
     * @param {?} nrfNestedControl
     */
    function NrfNestedControlContext(formControl, formGroup, nrfNestedControl) {
        this.formControl = formControl;
        this.formGroup = formGroup;
        this.nrfNestedControl = nrfNestedControl;
        this.$implicit = formControl;
    }
    return NrfNestedControlContext;
}());
export { NrfNestedControlContext };
function NrfNestedControlContext_tsickle_Closure_declarations() {
    /** @type {?} */
    NrfNestedControlContext.prototype.$implicit;
    /** @type {?} */
    NrfNestedControlContext.prototype.formControl;
    /** @type {?} */
    NrfNestedControlContext.prototype.formGroup;
    /** @type {?} */
    NrfNestedControlContext.prototype.nrfNestedControl;
}
/**
 * This directive control nested inputs and sets values on the Original Model set at {\@link NrfFormDirective#nrfEntity}
 *
 * #### Given an nrfEntity on the controller:
 * ```typescript
 * this.userModel = {
 *    firstName: 'John',
 *    address: {
 *        street: 'Carnaby Street'
 *    }
 * };
 * ```
 *
 * #### Use it on the form
 * ```html
 * <form [nrfEntity]="userModel">
 *   <div class="form-group">
 *      <label for="name">Name:</label>
 *
 *      <input
 *        id="name"
 *        class="form-control"
 *        nrfModel="userModel.name" // TODO fix this example
 *      />
 *   </div>
 * </form>
 * ```
 */
var NrfNestedControlDirective = /** @class */ (function () {
    /**
     * @param {?} modelSetter
     * @param {?} nrfForm
     * @param {?} templateRef
     * @param {?} viewContainerRef
     * @param {?} __4
     */
    function NrfNestedControlDirective(modelSetter, nrfForm, templateRef, viewContainerRef, _a) {
        var nativeElement = _a.nativeElement;
        this.modelSetter = modelSetter;
        this.nrfForm = nrfForm;
        this.templateRef = templateRef;
        this.viewContainerRef = viewContainerRef;
        this.isRegisteredToFormControl = false;
        this.isDestroyed = false;
        this.inputEl = nativeElement;
    }
    /**
     * Register this input component to its parent form [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     *
     * And starts to emit the input's value when it changes.
     * @return {?}
     */
    NrfNestedControlDirective.prototype.ngOnInit = function () {
        this.modelPath = this.getModelPathWithoutFirstPart();
        this.registerToFormGroup();
        this.subscribeToValueChanges();
        this.showViewContent();
    };
    /**
     * @return {?}
     */
    NrfNestedControlDirective.prototype.ngOnDestroy = function () {
        this.isDestroyed = true;
        if (this.parentFormGroup) {
            this.parentFormGroup.removeControl(this.nrfModelName);
        }
    };
    /**
     * @return {?}
     */
    NrfNestedControlDirective.prototype.showViewContent = function () {
        var /** @type {?} */ context = new NrfNestedControlContext(this.formControl, this.parentFormGroup, this);
        this.viewContainerRef.createEmbeddedView(this.templateRef, context);
    };
    /**
     * Register this input to its parent [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     * to enable validations and data manipulation
     * @return {?}
     */
    NrfNestedControlDirective.prototype.registerToFormGroup = function () {
        if (this.isRegisteredToFormControl) {
            return;
        }
        var /** @type {?} */ formGroup = this.getFormGroup();
        var /** @type {?} */ formControl = this.getFormControl(formGroup);
        this.formControl = formControl;
        this.inputEl.value = formControl.value || '';
        if (formGroup) {
            this.isRegisteredToFormControl = true;
            this.parentFormGroup = formGroup;
            formGroup.addControl(this.nrfModelName, formControl);
        }
    };
    /**
     * Verify if this input is inside a [NrfFormDirective]{\@link NrfFormDirective}
     * and return its [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     *
     * Otherwise a new empty [FormGroup]{\@link https://angular.io/api/forms/FormGroup}
     * @return {?}
     */
    NrfNestedControlDirective.prototype.getFormGroup = function () {
        if (this.nrfForm) {
            return this.nrfForm.formGroup;
        }
        return new FormGroup({});
    };
    /**
     * Verify if the [FormGroup]{\@link https://angular.io/api/forms/FormGroup} has an control with the current name and return it.
     * Otherwise return a new [FormControl]{\@link https://angular.io/api/forms/FormControl}
     * @param {?} formGroup
     * @return {?}
     */
    NrfNestedControlDirective.prototype.getFormControl = function (formGroup) {
        var /** @type {?} */ formControl = formGroup && /** @type {?} */ (formGroup.get(this.nrfModelName));
        if (!formControl) {
            formControl = this.getNewFormControl();
        }
        return formControl;
    };
    /**
     * Instantiate a new [FormControl]{\@link https://angular.io/api/forms/FormControl} and return it
     * @return {?}
     */
    NrfNestedControlDirective.prototype.getNewFormControl = function () {
        var /** @type {?} */ initialValue = this.getInitialValue();
        return new FormControl(initialValue || null);
    };
    /**
     * Subscribe to [valueChanges]{\@link https://angular.io/api/forms/AbstractControl#valueChanges} and update the Entity value
     * @return {?}
     */
    NrfNestedControlDirective.prototype.subscribeToValueChanges = function () {
        var _this = this;
        this.formControl.valueChanges
            .pipe(takeWhile(function () { return !_this.isDestroyed; }))
            .subscribe(function (newValue) { return _this.setModelValue(newValue); });
    };
    /**
     * Return the dot notation path of the nrfModelData, without the first part,
     * because it is the rfModelData itself.
     * @return {?}
     */
    NrfNestedControlDirective.prototype.getModelPathWithoutFirstPart = function () {
        return this.nrfModelName.substr(this.nrfModelName.indexOf('.') + 1);
    };
    /**
     * Get the value from the [nrfEntity]{\@link NrfFormDirective#nrfEntity}
     * @return {?}
     */
    NrfNestedControlDirective.prototype.getInitialValue = function () {
        return this.modelSetter.getValue(this.modelPath, this.nrfForm.nrfEntity);
    };
    /**
     * Set the value to the [formData]{\@link NrfFormDirective#formData}
     * @param {?} newValue
     * @return {?}
     */
    NrfNestedControlDirective.prototype.setModelValue = function (newValue) {
        return this.modelSetter.setValue(this.modelPath, newValue || '', this.nrfForm.formData);
    };
    /**
     * @param {?} input
     * @return {?}
     */
    NrfNestedControlDirective.prototype.onModelChange = function (input) {
        this.formControl.setValue(input.value, { emitModelToViewChange: false });
    };
    return NrfNestedControlDirective;
}());
export { NrfNestedControlDirective };
NrfNestedControlDirective.decorators = [
    { type: Directive, args: [{
                selector: '[nrfNestedControl]',
                exportAs: 'nrfNestedControl',
            },] },
];
/** @nocollapse */
NrfNestedControlDirective.ctorParameters = function () { return [
    { type: NrfModelSetterService, },
    { type: NrfFormDirective, decorators: [{ type: Optional },] },
    { type: TemplateRef, },
    { type: ViewContainerRef, },
    { type: ElementRef, },
]; };
NrfNestedControlDirective.propDecorators = {
    "nrfModelName": [{ type: Input, args: ['nrfNestedControl',] },],
    "onModelChange": [{ type: HostListener, args: ['input', ['$event.target'],] },],
};
function NrfNestedControlDirective_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    NrfNestedControlDirective.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    NrfNestedControlDirective.ctorParameters;
    /** @type {!Object<string,!Array<{type: !Function, args: (undefined|!Array<?>)}>>} */
    NrfNestedControlDirective.propDecorators;
    /**
     * The dot notation full name of the nrfEntity
     * @type {?}
     */
    NrfNestedControlDirective.prototype.nrfModelName;
    /** @type {?} */
    NrfNestedControlDirective.prototype.isRegisteredToFormControl;
    /** @type {?} */
    NrfNestedControlDirective.prototype.isDestroyed;
    /** @type {?} */
    NrfNestedControlDirective.prototype.parentFormGroup;
    /** @type {?} */
    NrfNestedControlDirective.prototype.formControl;
    /** @type {?} */
    NrfNestedControlDirective.prototype.modelPath;
    /** @type {?} */
    NrfNestedControlDirective.prototype.inputEl;
    /** @type {?} */
    NrfNestedControlDirective.prototype.modelSetter;
    /** @type {?} */
    NrfNestedControlDirective.prototype.nrfForm;
    /** @type {?} */
    NrfNestedControlDirective.prototype.templateRef;
    /** @type {?} */
    NrfNestedControlDirective.prototype.viewContainerRef;
}
//# sourceMappingURL=nested-control.directive.js.map
