import { EventEmitter, OnDestroy, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
export interface NrfSubmitData {
    nrfForm: NrfFormDirective;
    formData: any;
    entity: any;
    formGroup: FormGroup;
    event: Event;
}
/**
 * A component to abstract the form implementation
 *
 * It holds the validation and the submit logic
 *
 * @example
 * <form nrfForm (nrfSubmit)="aCallback($event)">
 *  // Inputs
 * </form>
 */
export declare class NrfFormDirective implements OnInit, OnDestroy {
    /**
     * Represents an Entity Model that comes from Database
     */
    nrfEntity: any;
    /**
     * A function that will be called when the form is valid and a submit event is triggered
     * by a button click or programmatically.
     */
    nrfSubmit: EventEmitter<NrfSubmitData>;
    /**
     * Form group will hold all inputs within this form.
     *
     * Every input should register itself to this form group.
     *
     * The data in this form group will not be sent to backend, just form validations and input management.
     */
    formGroup: FormGroup;
    /**
     * The form data representing all inputs
     */
    formData: any;
    /**
     * @ignore
     */
    constructor();
    /**
     * Init an empty nrfEntity, in case none where provided.
     */
    ngOnInit(): void;
    ngOnDestroy(): void;
    /**
     * # Should not be called directly!
     * This method wraps the form validation and call [onSubmit]{@link NrfFormDirective#nrfSubmit}
     */
    formSubmitWrapper($event: Event): void;
    cloneDeep(target: any | any[]): any;
}
