/**
 * Class to abstract get and set values from an object or an array
 * using dot-notation string as the path to the property
 *
 */
export declare class NrfModelSetterService {
    /**
     * Get a value using a dot-notation string to find the nested property
     *
     * @param path Dot notation of the value to be get
     * @param model An object or an Array
     * @param separator An optional param to specify a different separator, default '.'
     *
     * ```typescript
     * const nrfEntity = {
     *    user: {
     *        name: 'John Doe'
     *    }
     * };
     * nestedProps.getValue('user.name', nrfEntity);
     * // => 'John Doe'
     *
     * nestedProps.getValue('user.email', nrfEntity);
     * // => null
     * ```
     */
    getValue(path: string, model: any | any[], separator?: string): any | null;
    /**
     * @ignore
     *
     * Return the value associated to the given key.
     *
     * If the key is empty, the last element of an array is returned
     */
    private piecesReducer(obj, key);
    /**
     * Set a value using a dot-notation string.
     *
     * If the property does not exists, the entire path will be created.
     *
     * @param path Dot notation of the value to be get
     * @param value The value to be set
     * @param model An object or an Array
     * @param separator An optional param to specify a different separator, default '.'
     *
     * ```typescript
     * nestedProps.setValue('user.age', 35, { user: { name: 'John Doe' } });
     * ```
     */ setValue(path: string, value: any, model: object | any[], separator?: string): void;
    /**
     * Get the final property of the dot notation path.
     * This will be the property that will receive the new value
     */
    private getTargetPropToSet;
    /**
     * @ignore
     * Internal function to define if a given key is for an array or an object
     */
    private isArrayKey(key);
    /**
     * Generate the dot-notation path as an Array to iterate over.
     */
    private generatePathPieces(path, separator);
}
